"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseLineCounts = exports.getCommentTriggers = void 0;
const options_1 = require("../options");
const comments_1 = require("./comments");
const mappedCommentTriggers = new WeakMap();
function getCommentTriggers(key, debug) {
    const alreadyExisting = mappedCommentTriggers.get(key);
    if (!alreadyExisting) {
        return setCommentTriggers(key, debug);
    }
    return alreadyExisting;
}
exports.getCommentTriggers = getCommentTriggers;
function setCommentTriggers(rootNode, debug) {
    // parse comments only on the root node so it only happens once
    const comments = (0, comments_1.extractComments)(rootNode);
    if (debug) {
        console.info({ comments });
    }
    const commentTriggers = comments.reduce((accum, currentComment) => {
        var _a;
        const commentText = (_a = currentComment.value) === null || _a === void 0 ? void 0 : _a.replace(/\n/g, ' ');
        if (!currentComment.loc) {
            throw new Error(`Cannot read line location for comment ${currentComment.value}`);
        }
        const lineCounts = getLineCounts(commentText, debug);
        if (lineCounts.length) {
            accum.lineCounts[currentComment.loc.end.line] = lineCounts;
        }
        const wrapThreshold = getWrapThreshold(commentText);
        if (wrapThreshold != undefined) {
            accum.wrapThresholds[currentComment.loc.end.line] = wrapThreshold;
        }
        return accum;
    }, {
        lineCounts: {},
        wrapThresholds: {},
    });
    // save to a map so we don't have to recalculate these every time
    mappedCommentTriggers.set(rootNode, commentTriggers);
    return commentTriggers;
}
function getWrapThreshold(commentText) {
    if (commentText === null || commentText === void 0 ? void 0 : commentText.toLowerCase().includes(options_1.elementWrapThreshold)) {
        const thresholdValue = Number(commentText.toLowerCase().replace(options_1.untilWrapThresholdRegExp, '').trim());
        if (isNaN(thresholdValue)) {
            return undefined;
        }
        else {
            return thresholdValue;
        }
    }
    else {
        return undefined;
    }
}
function parseLineCounts(input, debug) {
    if (!input) {
        return [];
    }
    const split = input
        .toLowerCase()
        .replace(options_1.untilLineTriggerRegExp, '')
        .replace(/,/g, '')
        .split(' ')
        .filter((entry) => !!entry);
    const firstSplit = split[0];
    if (firstSplit === '[') {
        split.splice(0, 1);
    }
    else if (firstSplit === null || firstSplit === void 0 ? void 0 : firstSplit.startsWith('[')) {
        split[0] = firstSplit.replace(/^\[/, '');
    }
    const lastSplitIndex = split.length - 1;
    const lastSplit = split[lastSplitIndex];
    if (lastSplit === ']') {
        split.splice(split.length - 1, 1);
    }
    else if (lastSplit === null || lastSplit === void 0 ? void 0 : lastSplit.endsWith(']')) {
        split[lastSplitIndex] = lastSplit.replace(/\]$/, '');
    }
    const numbers = split.map((entry) => entry && !!entry.trim().match(/^\d+$/) ? Number(entry.trim()) : NaN);
    const invalidNumbers = numbers
        .map((entry, index) => ({ index, entry, original: split[index] }))
        .filter((entry) => {
        return isNaN(entry.entry);
    });
    if (invalidNumbers.length) {
        if (debug) {
            console.error(invalidNumbers.map((entry) => {
                var _a, _b, _c;
                return ({
                    index: entry.index,
                    original: entry.original,
                    parsed: entry,
                    split,
                    input,
                    numbers,
                    trim: (_a = entry.original) === null || _a === void 0 ? void 0 : _a.trim(),
                    match: (_b = entry.original) === null || _b === void 0 ? void 0 : _b.trim().match(/^\d+$/),
                    matched: !!((_c = entry.original) === null || _c === void 0 ? void 0 : _c.trim().match(/^\d+$/)),
                });
            }));
        }
        console.error(`Invalid number(s) for elements per line option/comment: ${invalidNumbers
            .map((entry) => entry.original)
            .join()}`);
        return [];
    }
    return numbers;
}
exports.parseLineCounts = parseLineCounts;
function getLineCounts(commentText, debug) {
    if (commentText === null || commentText === void 0 ? void 0 : commentText.toLowerCase().includes(options_1.elementsPerLineTrigger)) {
        return parseLineCounts(commentText, debug);
    }
    else {
        return [];
    }
}
